from __future__ import annotations

import urllib.parse
from typing import Tuple, List

import requests
import json

from datetime import datetime

from .base import BaseLolzAPI
from pylolzapi.types.user import User
from pylolzapi.types.payment import Operation
from pylolzapi.types.items import Item
from pylolzapi.utils.exceptions import LolzAPIError


class LZTApi(BaseLolzAPI):
    def __init__(self,
                 token: str = None,
                 client_id: str = None,
                 client_secret: str = None,
                 scope: list[str] = None
                 ) -> LZTApi:
        super(LZTApi, self).__init__(token, client_id, client_secret, scope)
        self._session = requests.session()
        self._session.headers = {"Authorization": f"Bearer {self._token}"}

        self.user_info: User = self.me()

    def __request(self, method, *args, **kwargs) -> requests.Response:
        response = method(*args, **kwargs)
        try:
            resp_json = response.json()
        except json.decoder.JSONDecodeError:
            raise LolzAPIError(response.text.split('<h1>')[1].split('</h1>')[0])

        if resp_json.get("errors") or resp_json.get("error"):
            if resp_json.get("error_description", False) is not False:
                raise LolzAPIError(resp_json["error_description"])
            else:
                raise LolzAPIError(",".join(resp_json["errors"]))

        return response

    def _get(self, url: str, params=None) -> dict:
        if params is None:
            params = dict()

        return self.__request(self._session.get,
                              self._base_url + url,
                              params=params).json()

    def _post(self, url: str, data=None) -> dict:
        if data is None:
            data = dict()

        return self.__request(self._session.post,
                              self._base_url + url,
                              data=data).json()

    def _delete(self, url: str, data=None) -> dict:
        if data is None:
            data = dict()

        return self.__request(self._session.delete,
                              self._base_url + url,
                              data=data).json()

    def me(self) -> User:
        """Отображает информацию о вашем профиле."""
        return User.parse_obj(self._get("users/me")["user"])

    def market_payments(self,
                        payment_type: str = None,
                        pmin: int = None,
                        pmax: int = None,
                        receiver: str = None,
                        sender: str = None,
                        start_date: datetime = None,
                        end_date: datetime = None,
                        wallet: str = None,
                        comment: str = None,
                        is_hold: str = None
                        ) -> tuple[list[Operation], dict]:
        """
        Выводит список транзакций на аккаунте.
        :param payment_type: Тип операции. Разрешенные типы операций: income, cost, refilled_balance,
        withdrawal_balance, paid_item, sold_item, money_transfer, receiving_money, internal_purchase, claim_hold
        :param pmin: Минимальная стоимость операции
        :param pmax: Максимальная стоимость операции
        :param receiver: Имя пользователя, который получает от вас деньги
        :param sender: Имя пользователя, который отправил вам деньги
        :param start_date: Дата начала операции (формат даты RFC 3339)
        :param end_date: Дата окончания операции (формат даты RFC 3339)
        :param wallet: Кошелек, который используется для денежных выплат
        :param comment: Комментарий для денежных переводов
        :param is_hold: Отображение операций удержания
        :return: dict
        """
        data = dict()

        if payment_type:
            data['type'] = payment_type
        if pmin:
            data['pmin'] = pmin
        if pmax:
            data['pmax'] = pmax
        if receiver:
            data['receiver'] = receiver
        if sender:
            data['sender'] = sender
        if start_date:
            data['startDate'] = start_date
        if end_date:
            data['endDate'] = end_date
        if wallet:
            data['wallet'] = wallet
        if comment:
            data['comment'] = comment
        if is_hold:
            data['is_hold'] = is_hold

        resp = self._get(f'market/user/{self.user_info.user_id}/payments', params=data)

        return [Operation.parse_obj(resp["payments"][operation]) for operation in resp["payments"]], resp

    def market_list(self, category: str = None,
                    pmin: int = None,
                    pmax: int = None,
                    title: str = None,
                    parse_sticky_items: str = None,
                    optional: dict = None
                    ) -> tuple[list[Item], dict]:
        """
        Получить все последние аккаунты маркета.
        :param category: Категория на маркете
        :param pmin: Минимальная цена для аккаунта
        :param pmax: Максимальная цена для аккаунта
        :param title: Название аккаунта
        :param parse_sticky_items: Условие для разбора параметров
        :param optional: Получить параметры URL-адреса из market
        :return:
        """
        if category:
            data = dict()

            if title:
                data['title'] = title
            if pmin:
                data['pmin'] = pmin
            if pmax:
                data['pmax'] = pmax
            if parse_sticky_items:
                data['parse_sticky_items'] = parse_sticky_items
            if optional:
                data = {**data, **optional}

            resp = self._get(f'market/{category}', params=data)
        else:
            resp = self._get('market')

        return [Item.parse_obj(i) for i in resp["items"]], resp

    def market_fave(self) -> dict:
        """
        Получить свои избранные товары
        :return: dict
        """
        return self._get(f'market/fave')

    def market_viewed(self) -> [Item, dict]:
        """
        Получить свои просмотренные товары.
        :return:
        """
        resp = self._get(f'market/viewed')
        return [Item.parse_obj(i) for i in resp["items"]], resp

    def market_item(self, item: int) -> Item:
        """
        Показывает информацию об аккаунте на маркете.
        :param item: Item ID
        :return:
        """
        return Item.parse_obj(self._get(f'market/{item}'))

    def market_reserve(self, item: int, price: int = None) -> dict:
        """
        Резервирует аккаунт.
        :param item: ID аккаунта
        :param price: Ваша Цена за аккаунт, по умолчанию указанная цена
        :return:
        """
        price = price if price is not None else (self.market_item(item)).price
        return self._post(f'market/{item}/reserve', data={'price': price})

    def market_cancel_reserve(self, item: int) -> dict:
        """
        Отменяет резерв аккаунта.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/cancel-reserve')

    def market_check_account(self, item: int) -> dict:
        """
        Проверяет аккаунт.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/check-account')

    def market_confirm_buy(self, item: int) -> dict:
        """
        Подтверждение покупки.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/confirm-buy')

    def market_fast_buy(self, item: int) -> dict:
        """
        Автоматическая проверка и покупка аккаунта.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/fast-buy')

    def market_get_email(self, item: int, email: str):
        """
        Получить код подтверждения с почты маркета.
        :param item: ID аккаунта
        :param email: Почта аккаунта
        :return:
        """
        return self._get(f'market/{item}/email-code', {'email': email})

    def market_refuse_guarantee(self, item: int) -> dict:
        """
        Отказаться от гарантии.
        :param item: ID Аккаунта
        :return: dict
        """
        return self._post(f'market/{item}/refuse-guarantee')

    def market_change_password(self, item: int) -> dict:
        """
        Изменить пароль аккаунта.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/change-password')

    def market_delete(self, item: int, reason: str):
        """
        Удалить свой аккаунт с маркета.
        :param item: ID аккаунта
        :param reason: Причина удаления
        :return:
        """
        return self._delete(f'market/{item}/delete', {'reason': reason})

    def market_bump(self, item: int):
        """
        Поднять аккаунт.
        :param item: ID аккаунта
        :return:
        """
        return self._post(f'market/{item}/bump')



from pydantic import BaseModel


class WTRankInfo(BaseModel):
    title: str
    img: str


class TelegramGroupCounters(BaseModel):
    chats: int | None
    channels: int | None
    conversations: int | None
    admin: int | None


class BumpSettings(BaseModel):
    canBumpItem: bool | None
    canBumpItemGlobally: bool | None
    errorPhrase: bool | None


class Reserve(BaseModel):
    reserve_user_id: int | None
    reserve_date: int | None


class Item(BaseModel):
    item_id: int | None
    item_state: str | None
    category_id: int | None
    published_date: int | None
    title: str | None
    description: str | None
    price: int | None
    update_stat_date: int | None
    refreshed_date: int | None
    view_count: int | None
    is_sticky: int | None
    item_origin: str | None
    extended_guarantee: int | None
    nsb: int | None
    allow_ask_discount: int | None
    title_en: str | None
    description_en: str | None
    email_type: str | None
    is_reserved: int | None
    item_domain: str | None
    isIgnored: bool | None
    canOpenItem: bool | None
    canCloseItem: bool | None
    canEditItem: bool | None
    canDeleteItem: bool | None
    canStickItem: bool | None
    canUnstickItem: bool | None
    bumpSettings: BumpSettings | None
    canBumpItem: bool | None
    canBuyItem: bool | None
    rub_price: int | None
    price_currency: str | None
    canValidateAccount: bool | None
    canResellItemAfterPurchase: bool | None
    canViewAccountLink: bool | None
    accountLink: str | None
    note_text: str | None
    tags: list | dict | None
    reserve: Reserve | None
    description_html: str | None
    description_html_en: str | None


class WarThunderItem(Item):
    wt_item_id: int | None
    wt_id: int | None
    wt_nick: str | None
    wt_reg_time_gaijin: int | None
    wt_reg_time_wt: int | None
    wt_last_play: int | None
    wt_email_verified: int | None
    wt_phone_verified: int | None
    wt_played: int | None
    wt_wins: int | None
    wt_exp: int | None
    wt_rank: int | None
    wt_eliteUnits: int | None
    wt_premium: int
    wt_gold: int | None
    wt_silver: int | None
    wt_win_count_percents: int | None
    wt_rank_info: WTRankInfo | None


class TelegramItem(Item):
    telegram_item_id: int | None
    telegram_country: str | None
    telegram_last_seen: int | None
    telegram_scam: int | None
    telegram_verified: int | str | None
    telegram_premium: int | None
    telegram_password: int | None
    telegram_premium_expires: int | None
    telegram_spam_block: int | None
    telegram_group_counters: TelegramGroupCounters | None
    telegram_admin_groups: list | None



from __future__ import annotations

from . import types
from . import utils
from .api.sync import LZTApi

__all__ = (
    'LZTApi',
    'types',
    'utils'
)

__version__ = '0.0.1'
__api_version__ = '0.1'



from pydantic import BaseModel, validator


class Data(BaseModel):
    user_id: int | None
    username: str | None
    includeFee: bool | None
    finishAmount: int | None
    status: str | None
    comment: str | None
    is_banned: int | None
    display_style_group_id: int | None
    uniq_username_css: str | None
    avatar_date: int | None
    user_group_id: int | None
    secondary_group_ids: int | str | None
    bool_status: bool | None


class Label(BaseModel):
    title: str


class User(BaseModel):
    user_id: int | None
    user_balance: int | None
    user_hold: int | None
    user_balance_with_hold: int | None


class Operation(BaseModel):
    operation_id: int
    operation_date: int
    operation_type: str
    outgoing_sum: int
    incoming_sum: int
    item_id: int
    wallet: str
    is_finished: int
    is_hold: int
    payment_system: str
    data: bool | Data | None
    hold_end_date: int
    api: int
    originalWallet: str | None
    payment_status: str
    supportLink: str | None
    canCancelBalanceTransfer: bool
    canCancelBalancePayout: bool
    canFinishBalanceTransfer: bool
    canFinishBalancePayout: bool
    label: Label
    user: User

    @validator("data", pre=True)
    def verify_is_boolean(cls, value):
        if isinstance(value, bool):
            return Data()



from __future__ import annotations

import urllib.parse


class BaseLolzAPI:
    def __init__(self,
                 token: str = None,
                 client_id: str = None,
                 client_secret: str = None,
                 scope: list[str] = None,
                 ) -> BaseLolzAPI:
        """
        https://zelenka.guru/account/api
        :param token: Токен
        :param client_id: Client ID (Временно не работает)
        :param client_secret: Client Secret (Временно не работает)
        :param scope: Разрешения (Временно не работает)
        """
        self.__token = token
        self._client_id = client_id
        self._client_secret = client_secret
        self._scope = "+".join(scope) if scope is not None else ""
        self._base_url = "https://api.zelenka.guru/"

    @property
    def _token(self):
        return self.__token

    @property
    def scopes(self):
        return self._scope

    @property
    def client_id(self):
        return self._client_id

    @_token.setter
    def _token(self, token):
        self.__token = token

    @staticmethod
    def transfer(username: str, amount: int = None, comment: str = None, hold: bool = None):
        url = f"https://zelenka.guru/market/balance/transfer?username={username}"

        if amount:
            url += f"&amount={amount}"
        if comment:
            comment = urllib.parse.quote(comment)
            url += f"&comment={comment}"
        if hold is not None:
            url += f"&hold={0 if hold is False else 1}"

        return url



class MarketCategories:
    Steam: int = 1
    VK: int = 2
    Origin: int = 3
    Warface: int = 4
    Uplay: int = 5
    SocialClub: int = 7
    Fortnite: int = 9
    Instagram: int = 10
    Battlenet: int = 11
    EpicGames: int = 12
    WoT: int = 14
    Supercell: int = 15
    WoTBlitz: int = 16
    GenshinImpact: int = 17
    Tarkov: int = 18
    VPN: int = 19
    TikTok: int = 20
    Discord: int = 22
    Cinema: int = 23
    Telegram: int = 24
    YouTube: int = 25



import setuptools
with open(r'E:\код\practice\works\libs\pyLolzApi\README.md', 'r', encoding='utf-8') as fh:
	long_description = fh.read()

setuptools.setup(
	name='pylolzapi',
	version='0.0.2',
	author='urlykkz',
	author_email='padreyonez@gmail.com',
	description='Python Wrapper for lolz.guru API',
	long_description=long_description,
	long_description_content_type='text/markdown',
	packages=['pylolzapi'],
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	install_requires=['requests~=2.28.1\n', 'DateTime~=4.7\n', 'pydantic~=1.9.1'],
	python_requires='>=3.6',
	package_data={
		"pylolzapi": ['api/*.py', 'utils/*.py', 'types/*.py']
	},
)



from pydantic import BaseModel


class Links(BaseModel):
    permalink: str
    detail: str
    avatar: str
    avatar_big: str
    avatar_small: str
    followers: str
    followings: str
    ignore: str
    timeline: str


class Permissions(BaseModel):
    edit: bool
    follow: bool
    ignore: bool
    profile_post: bool


class Field(BaseModel):
    id: str | int | None
    title: str | None
    description: str | None
    position: str | None
    is_required: bool | None
    value: str | None
    is_multi_choice: bool | None


class Group(BaseModel):
    user_group_id: int
    user_group_title: str
    is_primary_group: bool


class SelfPermissions(BaseModel):
    create_conversation: bool | None
    upload_attachment_conversation: bool | None


class EditPermissions(BaseModel):
    password: bool | None
    user_email: bool | None
    username: bool | None
    user_title: bool | None
    primary_group_id: bool | None
    secondary_group_ids: bool | None
    user_dob_day: bool | None
    user_dob_month: bool | None
    user_dob_year: bool | None
    fields: bool | None


class User(BaseModel):
    user_id: int
    username: str
    user_message_count: int
    user_register_date: int
    user_like_count: int
    short_link: str
    user_email: str
    user_unread_notification_count: int
    user_dob_day: int
    user_dob_month: int
    user_dob_year: int
    user_title: str
    user_is_valid: bool
    user_is_verified: bool
    user_is_followed: bool
    user_last_seen_date: int
    links: Links
    permissions: Permissions
    user_is_ignored: bool
    user_is_visitor: bool
    user_timezone_offset: int
    user_has_password: bool
    fields: list[Field]
    user_groups: list[Group]
    self_permissions: SelfPermissions
    edit_permissions: EditPermissions
